@@ -1,4 +1,4 @@
-/* Krb5 Auth Applet -- Acquire and release kerberos tickets
+/* Krb5 Auth Applet -- Acquire and release Kerberos tickets
  *
  * (C) 2009 Guido Guenther <agx@sigxcpu.org>
  *
@@ -33,7 +33,7 @@ enum ticket_columns {
 };
 
 
-GtkApplicationWindow *ka_main_window_create (KaApplet *applet, GtkBuilder *xml);
+GtkApplicationWindow *ka_main_window_create (KaApplet *applet);
 void ka_main_window_show (KaApplet *applet);
 void ka_main_window_hide (void);
 
