@@ -1,4 +1,4 @@
-/* Krb5 Auth Applet -- Acquire and release kerberos tickets
+/* Krb5 Auth Applet -- Acquire and release Kerberos tickets
  *
  * (C) 2009 Guido Guenther <agx@sigxcpu.org>
  *
@@ -45,7 +45,7 @@ typedef struct _KaPwDialogPrivate KaPwDialogPrivate;
 GType ka_pwdialog_get_type (void);
 
 /* public functions */
-KaPwDialog *ka_pwdialog_create (GtkBuilder *xml);
+KaPwDialog *ka_pwdialog_new (void);
 
 /* setup everything for the next prompting */
 void ka_pwdialog_setup (KaPwDialog *pwdialog, const gchar *krb5prompt,
