@@ -21,7 +21,7 @@
 #include "ka-tools.h"
 
 void
-ka_show_help (GdkScreen* screen, const char* chapter, GtkWindow* window)
+ka_show_help (GtkWindow* window, const char* chapter)
 {
   GError *error = NULL;
   const char *section = "";
@@ -30,10 +30,9 @@ ka_show_help (GdkScreen* screen, const char* chapter, GtkWindow* window)
   if (chapter)
 	section = chapter;
 
-  url = g_strdup_printf("ghelp:krb5-auth-dialog%s", section);
-
-  gtk_show_uri (screen, url, gtk_get_current_event_time (), &error);
+  url = g_strdup_printf("help:krb5-auth-dialog%s", section);
 
+  gtk_show_uri_on_window (window, url, gtk_get_current_event_time (), &error);
   if (error) {
       GtkWidget *message_dialog;
 
@@ -75,7 +74,7 @@ ka_show_about (KaApplet *applet)
                            "logo-icon-name", "krb-valid-ticket",
                            "copyright",
                            "Copyright (C) 2004,2005,2006 Red Hat, Inc.,\n"
-                           "2008-2011 Guido Günther",
+                           "2008-2011,2014 Guido Günther",
                            "website-label", PACKAGE " website",
                            "website",
                            "https://honk.sigxcpu.org/piki/projects/krb5-auth-dialog/",
